@@ -6,8 +6,12 @@
  */
 
 use super::*;
+use hanower::SIGNIFICANT;
 use structopt::clap::AppSettings::AllowLeadingHyphen;
 
+// Helps handle Rust's odd floating point support
+const WORKAROUND: f64 = SIGNIFICANT / 1e3;
+
 /// Finds the `count` number of intervals in a range from `low` to `high`
 #[derive(Debug, StructOpt)]
 #[structopt(setting = AllowLeadingHyphen)]
@@ -38,8 +42,16 @@ impl Runner for SubComInterval {
 
         let interval = hanower::Interval::new(low, high, self.count)?;
 
-        for number in interval.intervals().map(|f| f.round() as i64) {
-            write!(dst, "{} ", number)?;
+        //for number in interval.intervals().map(|f| f.round() as i64) {
+        for number in interval.intervals() {
+            let res = number.abs() - number.round().abs();
+            // dbg!(res);
+            // Work around for Rust's atrocious floating point support
+            if (res / number).abs() < WORKAROUND {
+                write!(dst, "{:.0} ", number.round())?
+            } else {
+                write!(dst, "{} ", number)?
+            }
         }
         writeln!(dst)?;
 
